@@ -110,6 +110,7 @@ gdata_service_class_init (GDataServiceClass *klass)
 
 	klass->service_name = "xapi";
 	klass->authentication_uri = "https://www.google.com/accounts/ClientLogin";
+	klass->api_version = "2";
 	klass->feed_type = GDATA_TYPE_FEED;
 	klass->parse_authentication_response = real_parse_authentication_response;
 	klass->append_query_headers = real_append_query_headers;
@@ -324,7 +325,7 @@ real_append_query_headers (GDataService *self, SoupMessage *message)
 	}
 
 	/* Set the GData-Version header to tell it we want to use the v2 API */
-	soup_message_headers_append (message->request_headers, "GData-Version", "2");
+	soup_message_headers_append (message->request_headers, "GData-Version", GDATA_SERVICE_GET_CLASS (self)->api_version);
 }
 
 static void
@@ -332,28 +333,27 @@ real_parse_error_response (GDataService *self, GDataServiceError error_type, gui
 			   gint length, GError **error)
 {
 	/* See: http://code.google.com/apis/gdata/docs/2.0/reference.html#HTTPStatusCodes */
-
 	switch (status) {
 		case 400:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
 				     /* Translators: the parameter is an error message returned by the server. */
-				     _("Invalid request URI or header, or unsupported nonstandard parameter: %s"), reason_phrase);
+				     _("Invalid request URI or header, or unsupported nonstandard parameter: %s"), response_body);
 			return;
 		case 401:
 		case 403:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
 				     /* Translators: the parameter is an error message returned by the server. */
-				     _("Authentication required: %s"), reason_phrase);
+				     _("Authentication required: %s"), response_body);
 			return;
 		case 404:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_NOT_FOUND,
 				     /* Translators: the parameter is an error message returned by the server. */
-				     _("The requested resource was not found: %s"), reason_phrase);
+				     _("The requested resource was not found: %s"), response_body);
 			return;
 		case 409:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_CONFLICT,
 				     /* Translators: the parameter is an error message returned by the server. */
-				     _("The entry has been modified since it was downloaded: %s"), reason_phrase);
+				     _("The entry has been modified since it was downloaded: %s"), response_body);
 			return;
 		case 500:
 		default:
@@ -365,23 +365,33 @@ real_parse_error_response (GDataService *self, GDataServiceError error_type, gui
 	switch (error_type) {
 		case GDATA_SERVICE_ERROR_WITH_INSERTION:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_INSERTION,
-				     /* Translators: the first parameter is a HTTP status, and the second is an error message returned by the server. */
-				     _("Error code %u when inserting an entry: %s"), status, reason_phrase);
+				     /* Translators: the first parameter is an HTTP status, and the second is an error message returned by the server. */
+				     _("Error code %u when inserting an entry: %s"), status, response_body);
 			break;
 		case GDATA_SERVICE_ERROR_WITH_UPDATE:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_UPDATE,
-				     /* Translators: the first parameter is a HTTP status, and the second is an error message returned by the server. */
-				     _("Error code %u when updating an entry: %s"), status, reason_phrase);
+				     /* Translators: the first parameter is an HTTP status, and the second is an error message returned by the server. */
+				     _("Error code %u when updating an entry: %s"), status, response_body);
 			break;
 		case GDATA_SERVICE_ERROR_WITH_DELETION:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_DELETION,
-				     /* Translators: the first parameter is a HTTP status, and the second is an error message returned by the server. */
-				     _("Error code %u when deleting an entry: %s"), status, reason_phrase);
+				     /* Translators: the first parameter is an HTTP status, and the second is an error message returned by the server. */
+				     _("Error code %u when deleting an entry: %s"), status, response_body);
 			break;
 		case GDATA_SERVICE_ERROR_WITH_QUERY:
 			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_QUERY,
-				     /* Translators: the first parameter is a HTTP status, and the second is an error message returned by the server. */
-				     _("Error code %u when querying: %s"), status, reason_phrase);
+				     /* Translators: the first parameter is an HTTP status, and the second is an error message returned by the server. */
+				     _("Error code %u when querying: %s"), status, response_body);
+			break;
+		case GDATA_SERVICE_ERROR_WITH_DOWNLOAD:
+			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_DOWNLOAD,
+				     /* Translators: the first parameter is an HTTP status, and the second is an error message returned by the server. */
+				     _("Error code %u when downloading: %s"), status, response_body);
+			break;
+		case GDATA_SERVICE_ERROR_WITH_UPLOAD:
+			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_UPLOAD,
+				     /* Translators: the first parameter is an HTTP status, and the second is an error message returned by the server. */
+				     _("Error code %u when uploading: %s"), status, response_body);
 			break;
 		default:
 			/* We should not be called with anything other than the above four generic error types */
@@ -404,6 +414,8 @@ authenticate_async_data_free (AuthenticateAsyncData *self)
 {
 	g_free (self->username);
 	g_free (self->password);
+	if (self->service != NULL)
+		g_object_unref (self->service);
 
 	g_slice_free (AuthenticateAsyncData, self);
 }
@@ -460,7 +472,7 @@ authenticate_thread (GSimpleAsyncResult *result, GDataService *service, GCancell
 	}
 
 	/* Update the authentication details held by the service */
-	data->service = service;
+	data->service = g_object_ref (service);
 	g_idle_add ((GSourceFunc) set_authentication_details_cb, data);
 }
 
@@ -495,6 +507,7 @@ gdata_service_authenticate_async (GDataService *self, const gchar *username, con
 	data = g_slice_new (AuthenticateAsyncData);
 	data->username = g_strdup (username);
 	data->password = g_strdup (password);
+	data->service = NULL; /* set in authenticate_thread() */
 
 	result = g_simple_async_result_new (G_OBJECT (self), callback, user_data, gdata_service_authenticate_async);
 	g_simple_async_result_set_op_res_gpointer (result, data, NULL);
@@ -1037,8 +1050,10 @@ gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *quer
 	g_return_val_if_fail (entry_type != G_TYPE_INVALID, NULL);
 
 	message = _gdata_service_query (self, feed_uri, query, cancellable, progress_callback, progress_user_data, error);
-	g_assert (message->response_body->data != NULL);
+	if (message == NULL)
+		return NULL;
 
+	g_assert (message->response_body->data != NULL);
 	klass = GDATA_SERVICE_GET_CLASS (self);
 	feed = _gdata_feed_new_from_xml (klass->feed_type, message->response_body->data, message->response_body->length, entry_type,
 					 progress_callback, progress_user_data, error);
@@ -1538,7 +1553,7 @@ gdata_service_delete_entry_finish (GDataService *self, GAsyncResult *async_resul
  * @cancellable: optional #GCancellable object, or %NULL
  * @error: a #GError, or %NULL
  *
- * Deletes @entry from the server. For more information about the concept of updating entries, see the
+ * Deletes @entry from the server. For more information about the concept of deleting entries, see the
  * <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/basics.html#DeletingEntry">online documentation</ulink> for the GData
  * protocol.
  *
@@ -1718,3 +1733,9 @@ gdata_service_get_password (GDataService *self)
 	g_return_val_if_fail (GDATA_IS_SERVICE (self), NULL);
 	return self->priv->password;
 }
+
+SoupSession *
+_gdata_service_get_session (GDataService *self)
+{
+	return self->priv->session;
+}
