@@ -30,6 +30,8 @@
 #include <gdata/gdata-access-handler.h>
 #include <gdata/gdata-access-rule.h>
 #include <gdata/gdata-parsable.h>
+#include <gdata/gdata-download-stream.h>
+#include <gdata/gdata-upload-stream.h>
 
 /* Namespaces */
 
@@ -42,6 +44,7 @@
 /* GData */
 #include <gdata/gd/gdata-gd-email-address.h>
 #include <gdata/gd/gdata-gd-im-address.h>
+#include <gdata/gd/gdata-gd-name.h>
 #include <gdata/gd/gdata-gd-organization.h>
 #include <gdata/gd/gdata-gd-phone-number.h>
 #include <gdata/gd/gdata-gd-postal-address.h>
